@@ -48,7 +48,7 @@ typedef struct cond_expr {
 #define COND_XOR	5	/* bool ^ bool */
 #define COND_EQ		6	/* bool == bool */
 #define COND_NEQ	7	/* bool != bool */
-#define COND_LAST	8
+#define COND_LAST	COND_NEQ
 	uint32_t expr_type;
 	uint32_t bool;
 	struct cond_expr *next;
@@ -100,6 +100,8 @@ extern cond_node_t *cond_node_find(policydb_t * p,
 				   cond_node_t * needle, cond_node_t * haystack,
 				   int *was_created);
 
+extern cond_node_t *cond_node_create(policydb_t * p, cond_node_t * node);
+
 extern cond_node_t *cond_node_search(policydb_t * p, cond_node_t * list,
 				     cond_node_t * cn);
 
